// src/react.tsx
import React from "react";
import { createContext, useContext, useRef } from "react";
import { useStore } from "zustand";

// src/store/index.ts
import { createStore } from "zustand";

// src/store/chains.ts
import { ApiPromise } from "@polkadot/api";
var createAPISlice = (props) => (set) => {
  const api = {};
  Object.entries(props.chains).forEach(([key, options]) => {
    const chainAPI = new ApiPromise(options);
    chainAPI.on("error", (e) => {
      console.error("Error on ", key, e);
      set((state) => ({
        error: {
          [key]: Error("TODO decode error"),
          ...state.error
        },
        ready: {
          [key]: false,
          ...state.ready
        }
      }));
    });
    chainAPI.on("disconnected", (e) => {
      console.error("Disconnected on", key, e);
      set((state) => ({
        error: {
          [key]: Error("TODO decode error on console"),
          ...state.error
        },
        connected: {
          [key]: false,
          ...state.connected
        },
        ready: {
          [key]: false,
          ...state.ready
        }
      }));
    });
    chainAPI.on("connected", () => {
      console.info("Connected on", key);
      set((state) => ({
        error: {
          [key]: void 0,
          ...state.error
        },
        connected: {
          [key]: true,
          ...state.connected
        }
      }));
    });
    chainAPI.on("ready", () => {
      console.info("Ready on", key);
      set((state) => ({
        ready: {
          [key]: true,
          ...state.ready
        },
        errors: {
          [key]: void 0,
          ...state.error
        }
      }));
    });
    api[key] = chainAPI;
  });
  return {
    api
  };
};

// src/store/wallet.ts
import { web3Accounts, web3Enable } from "@polkadot/extension-dapp";
var createWalletSlice = (props) => (set, get) => {
  const connect = async () => {
    set({ isConnecting: true });
    const extensions = await web3Enable("Polkadot Dapp Template");
    if (extensions.length !== 0) {
      const accounts = await web3Accounts();
      set({ accounts, isConnected: true });
      return accounts;
    }
    set({ isConnecting: false });
    return [];
  };
  const selectAccount = (address) => {
    set({
      selectedAccount: get().accounts.find((a) => a.address === address)
    });
  };
  if (props.eagerConnect) {
    void connect().then((accounts) => {
      if (accounts[0]) {
        selectAccount(accounts[0].address);
      }
    });
  }
  return {
    accounts: [],
    isConnected: false,
    isConnecting: false,
    selectedAccount: void 0,
    connect,
    selectAccount
  };
};

// src/store/index.ts
var createPolkadotDappStore = (initProps) => {
  const props = {
    eagerConnect: false,
    chains: {},
    ...initProps
  };
  return createStore()((...a) => ({
    ...createAPISlice(props)(...a),
    ...createWalletSlice(props)(...a)
  }));
};

// src/react.tsx
var PolkadotDappContext = createContext(
  null
);
var PolkadotDappProvider = ({ children, ...props }) => {
  const storeRef = useRef();
  if (!storeRef.current) {
    storeRef.current = createPolkadotDappStore(props);
  }
  return /* @__PURE__ */ React.createElement(PolkadotDappContext.Provider, { value: storeRef.current }, children);
};
function usePolkadotDapp(selector) {
  const store = useContext(PolkadotDappContext);
  if (!store)
    throw new Error("Missing PolkadotDappContext.Provider in the tree");
  return useStore(store, selector);
}
var useApi = (chain) => {
  return usePolkadotDapp((state) => state.api[chain]);
};
var useChain = useApi;
var useWallet = () => {
  return usePolkadotDapp((state) => ({
    accounts: state.accounts,
    connect: state.connect,
    isConnected: state.isConnected,
    isConnecting: state.isConnecting,
    selectAccount: state.selectAccount,
    selectedAccount: state.selectedAccount
  }));
};
export {
  PolkadotDappProvider,
  useChain,
  usePolkadotDapp,
  useWallet
};
//# sourceMappingURL=index.js.map